/*
 *************************************************************************
 * The contents of this file are subject to the Openbravo  Public  License
 * Version  1.1  (the  "License"),  being   the  Mozilla   Public  License
 * Version 1.1  with a permitted attribution clause; you may not  use this
 * file except in compliance with the License. You  may  obtain  a copy of
 * the License at http://www.openbravo.com/legal/license.html 
 * Software distributed under the License  is  distributed  on  an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific  language  governing  rights  and  limitations
 * under the License. 
 * The Original Code is Openbravo ERP. 
 * The Initial Developer of the Original Code is Openbravo SLU 
 * All portions are Copyright (C) 2010-2011 Openbravo SLU 
 * All Rights Reserved. 
 * Contributor(s):  ______________________________________.
 ************************************************************************
 */
package org.openbravo.client.kernel.reference;

import org.openbravo.base.model.Property;
import org.openbravo.client.kernel.KernelUtils;
import org.openbravo.model.ad.ui.Field;
import org.openbravo.service.json.JsonConstants;

/**
 * Base class of all foreign key/reference ui definitions.
 * 
 * @author mtaal
 */
public class ForeignKeyUIDefinition extends UIDefinition {

  @Override
  public String getParentType() {
    return "text";
  }

  @Override
  public String getFormEditorType() {
    return "OBFKItem";
  }

  @Override
  public String getFilterEditorType() {
    return "OBFKFilterTextItem";
  }

  @Override
  public String getFilterEditorProperties(Field field) {
    return ", filterOnKeypress: true" + super.getFilterEditorProperties(field);
  }

  @Override
  public String getGridFieldProperties(Field field) {
    Long length = field.getDisplayedLength();
    if (length == null || length == 0) {
      length = field.getColumn().getLength();
    }
    final Property prop = KernelUtils.getInstance().getPropertyFromColumn(field.getColumn());
    return ", width: isc.OBGrid.getDefaultColumnWidth(" + length + "), displayField: '"
        + getDisplayFieldName(field, prop) + "'," + "valueField: '" + prop.getName() + "'"
        + ", foreignKeyField: true" + super.getGridFieldProperties(field)
        + getShowHoverGridFieldSettings(field);
  }

  protected String getDisplayFieldName(Field field, Property prop) {
    return prop.getName() + "." + JsonConstants.IDENTIFIER;
  }

  protected String getSuperGridFieldProperties(Field field) {
    return super.getGridFieldProperties(field);
  }

  protected String getSuperGridFieldName(Field field) {
    return super.getGridFieldName(field);
  }
}
